@@ -31,7 +31,10 @@ from openleadr.messaging import create_message, parse_message, \
                                 validate_xml_schema, validate_xml_signature
 from openleadr import utils
 
+import tzlocal
+
 logger = logging.getLogger('openleadr')
+logger.setLevel(logging.INFO)
 
 
 class OpenADRClient:
@@ -41,7 +44,7 @@ class OpenADRClient:
     """
     def __init__(self, ven_name, vtn_url, debug=False, cert=None, key=None,
                  passphrase=None, vtn_fingerprint=None, show_fingerprint=True, ca_file=None,
-                 allow_jitter=True, ven_id=None):
+                 allow_jitter=True, ven_id=None, disable_signature=False):
         """
         Initializes a new OpenADR Client (Virtual End Node)
 
@@ -61,6 +64,7 @@ class OpenADRClient:
                             certificate.
         :param str ven_id: The ID for this VEN. If you leave this blank,
                            a VEN_ID will be assigned by the VTN.
+        :param bool disable_signature: Whether or not to sign outgoing messages using a public-private key pair in PEM format.
         """
 
         self.ven_name = ven_name
@@ -78,7 +82,7 @@ class OpenADRClient:
         self.report_requests = []               # Keep track of the report requests from the VTN
         self.incomplete_reports = {}            # Holds reports that are being populated over time
         self.pending_reports = asyncio.Queue()  # Holds reports that are waiting to be sent
-        self.scheduler = AsyncIOScheduler()
+        self.scheduler = AsyncIOScheduler(timezone=str(tzlocal.get_localzone()))
         self.client_session = None
         self.report_queue_task = None
 
@@ -109,7 +113,8 @@ class OpenADRClient:
         self._create_message = partial(create_message,
                                        cert=cert,
                                        key=key,
-                                       passphrase=passphrase)
+                                       passphrase=passphrase,
+                                       disable_signature=disable_signature)
 
     async def run(self):
         """
@@ -120,8 +125,8 @@ class OpenADRClient:
         self.loop = asyncio.get_event_loop()
         await self.create_party_registration(ven_id=self.ven_id)
 
-        if not self.ven_id:
-            logger.error("No VEN ID received from the VTN, aborting.")
+        if not self.registration_id:
+            logger.error("No RegistrationID received from the VTN, aborting.")
             await self.stop()
             return
 
@@ -172,7 +177,9 @@ class OpenADRClient:
                    report_specifier_id=None, r_id=None,
                    report_name=enums.REPORT_NAME.TELEMETRY_USAGE,
                    reading_type=enums.READING_TYPE.DIRECT_READ,
-                   report_type=enums.REPORT_TYPE.READING, sampling_rate=None, data_source=None,
+                   report_type=enums.REPORT_TYPE.READING,
+                   report_duration=None, report_dtstart=None,
+                   sampling_rate=None, data_source=None,
                    scale="none", unit=None, power_ac=True, power_hertz=50, power_voltage=230,
                    market_context=None, end_device_asset_mrid=None, report_data_source=None):
         """
@@ -201,6 +208,8 @@ class OpenADRClient:
         :param str report_name: An OpenADR name for this report (one of openleadr.enums.REPORT_NAME)
         :param str reading_type: An OpenADR reading type (found in openleadr.enums.READING_TYPE)
         :param str report_type: An OpenADR report type (found in openleadr.enums.REPORT_TYPE)
+        :param datetime.timedelta report_duration: The time span that can be provided in this report.
+        :param datetime.datetime report_dtstart: The earliest available data for this report (defaults to now).
         :param datetime.timedelta sampling_rate: The sampling rate for the measurement.
         :param str unit: The unit for this measurement.
         :param boolean power_ac: Whether the power is AC (True) or DC (False).
@@ -231,6 +240,16 @@ class OpenADRClient:
             raise ValueError(f"{scale} is not a valid scale. Valid options are "
                              f"{', '.join(enums.SI_SCALE_CODE.values)}")
 
+        if report_duration is None:
+            logger.warning("You did not provide a 'report_duration' parameter to 'add_report'. "
+                           "This parameter should indicate the size of the data buffer that "
+                           "can be built up. It will now default to 3600 seconds, which may "
+                           "or may not be appropriate for your use case.")
+            report_duration = timedelta(seconds=3600)
+
+        if report_dtstart is None:
+            report_dtstart = datetime.now(timezone.utc)
+
         if sampling_rate is None:
             sampling_rate = objects.SamplingRate(min_period=timedelta(seconds=10),
                                                  max_period=timedelta(hours=24),
@@ -298,7 +317,9 @@ class OpenADRClient:
             report = objects.Report(created_date_time=datetime.now(),
                                     report_name=report_name,
                                     report_specifier_id=report_specifier_id,
-                                    data_collection_mode=data_collection_mode)
+                                    data_collection_mode=data_collection_mode,
+                                    duration=report_duration,
+                                    dtstart=report_dtstart)
             self.reports.append(report)
 
         # Add the new report description to the report
@@ -362,20 +383,18 @@ class OpenADRClient:
         :param str transport_name: The transport name to use. Either 'simpleHttp' or 'xmpp'.
         :param str transport_address: Which public-facing address the server should use
                                       to communicate.
-        :param str ven_id: The ID for this VEN. If you leave this blank,
-                           a VEN_ID will be assigned by the VTN.
         """
         request_id = utils.generate_id()
         service = 'EiRegisterParty'
         payload = {'ven_name': self.ven_name,
+                   'ven_id': self.ven_id,
                    'http_pull_model': http_pull_model,
                    'xml_signature': xml_signature,
                    'report_only': report_only,
                    'profile_name': profile_name,
                    'transport_name': transport_name,
                    'transport_address': transport_address}
-        if ven_id:
-            payload['ven_id'] = ven_id
+
         message = self._create_message('oadrCreatePartyRegistration',
                                        request_id=request_id,
                                        **payload)
@@ -388,12 +407,30 @@ class OpenADRClient:
             logger.error(f"Got error on Create Party Registration: "
                          f"{status_code} {status_description}")
             return
-        self.ven_id = response_payload['ven_id']
-        self.registration_id = response_payload['registration_id']
-        self.poll_frequency = response_payload.get('requested_oadr_poll_freq',
-                                                   timedelta(seconds=10))
-        logger.info(f"VEN is now registered with ID {self.ven_id}")
-        logger.info(f"The polling frequency is {self.poll_frequency}")
+
+        if response_payload.get('registration_id'):
+            self.registration_id = response_payload['registration_id']
+        else:
+            logger.error("No registration ID received from the VTN during registration. "
+                         "Will assume that we are not or no longer registered.")
+            self.registration_id = None
+
+        if response_payload.get('ven_id'):
+            if self.ven_id and response_payload['ven_id'] != self.ven_id:
+                logger.warning(f"The venID that was received from the VTN {response_payload['ven_id']} "
+                               "did not match the venID the venID that was previously configured in the "
+                               f"OpenLEADR client ({self.ven_id}). Will update the venId in the OpenLEADR "
+                               "client to the value supplied by the VEN.")
+            self.ven_id = response_payload['ven_id']
+        else:
+            logger.error("No venID received from the VTN during registration. "
+                         "Will assume that we are not or no longer registered.")
+
+        if self.registration_id:
+            self.poll_frequency = response_payload.get('requested_oadr_poll_freq',
+                                                       timedelta(seconds=10))
+            logger.info(f"VEN is now registered with registration ID {self.registration_id} and venID {self.ven_id}.")
+            logger.info(f"The polling frequency is {self.poll_frequency}")
         return response_type, response_payload
 
     async def cancel_party_registration(self):
@@ -452,6 +489,9 @@ class OpenADRClient:
                    'reports': reports,
                    'report_request_id': 0}
 
+        for report in payload['reports']:
+            utils.setmember(report, 'report_request_id', 0)
+
         service = 'EiReport'
         message = self._create_message('oadrRegisterReport', **payload)
         response_type, response_payload = await self._perform_request(service, message)
@@ -461,10 +501,17 @@ class OpenADRClient:
             for report_request in response_payload['report_requests']:
                 await self.create_report(report_request)
 
+        # Send the oadrCreatedReport message
         message_type = 'oadrCreatedReport'
-        message_payload = {}
-
-        return message_type, message_payload
+        message_payload = {'pending_reports':
+                           [{'report_request_id': utils.getmember(report, 'report_request_id')}
+                            for report in self.report_requests]}
+        message = self._create_message(message_type,
+                                       response={'response_code': 200,
+                                                 'response_description': 'OK'},
+                                       ven_id=self.ven_id,
+                                       **message_payload)
+        response_type, response_payload = await self._perform_request(service, message)
 
     async def create_report(self, report_request):
         """
@@ -603,6 +650,10 @@ class OpenADRClient:
         outgoing_report.intervals = intervals
         logger.info(f"The number of intervals in the report is now {len(outgoing_report.intervals)}")
 
+        # Always set the dtstart of the report to the earliest datetime of any of the intervals
+        if outgoing_report.intervals:
+            outgoing_report.dtstart = min(interval.dtstart for interval in outgoing_report.intervals)
+
         # Figure out if the report is complete after this sampling
         if data_collection_mode == 'incremental' and report_back_duration is not None\
                 and report_back_duration > granularity:
@@ -677,6 +728,25 @@ class OpenADRClient:
         Placeholder for the on_register_report handler.
         """
 
+    async def on_cancel_party_registration(self, message):
+        # Update/Delete all the registration and reports information
+        self.registration_id = None
+        self.report_requests = None
+        self.reports = None
+        self.report_callbacks = None
+        self.report_requests = None
+        self.incomplete_reports = None
+        self.pending_reports = None
+        self.scheduler.remove_all_jobs()
+
+        response = {'response_code': 200,
+                    'response_description': 'OK',
+                    'request_id': message['request_id']}
+        message = self._create_message('oadrCanceledPartyRegistration', response=response)
+        service = 'EiRegisterParty'
+        response_type, response_payload = await self._perform_request(service, message)
+        logger.info(response_type, response_payload)
+
     ###########################################################################
     #                                                                         #
     #                             EMPTY RESPONSES                             #
@@ -701,7 +771,6 @@ class OpenADRClient:
 
     async def _perform_request(self, service, message):
         await self._ensure_client_session()
-        logger.debug(f"Client is sending {message}")
         url = f"{self.vtn_url}/{service}"
         try:
             async with self.client_session.post(url, data=message) as req:
@@ -710,7 +779,6 @@ class OpenADRClient:
                     logger.warning(f"Non-OK status {req.status} when performing a request to {url} "
                                    f"with data {message}: {req.status} {content.decode('utf-8')}")
                     return None, {}
-                logger.debug(content.decode('utf-8'))
         except aiohttp.client_exceptions.ClientConnectorError as err:
             # Could not connect to server
             logger.error(f"Could not connect to server with URL {self.vtn_url}:")
@@ -772,7 +840,8 @@ class OpenADRClient:
                 if asyncio.iscoroutine(result):
                     result = await result
                 results.append(result)
-                if event_status in (enums.EVENT_STATUS.COMPLETED, enums.EVENT_STATUS.CANCELLED):
+                if event_status in (enums.EVENT_STATUS.COMPLETED, enums.EVENT_STATUS.CANCELLED) \
+                        and event_id in self.responded_events:
                     self.responded_events.pop(event_id)
                 else:
                     self.responded_events[event_id] = result
@@ -797,6 +866,7 @@ class OpenADRClient:
                            and not utils.determine_event_status(event['active_period']) == 'completed']
 
         if len(event_responses) > 0:
+            logger.info(f"Total event_responses: {len(event_responses)}")
             response = {'response_code': 200,
                         'response_description': 'OK',
                         'request_id': message['request_id']}
@@ -834,11 +904,10 @@ class OpenADRClient:
             logger.info("The VTN required us to re-register. Calling the registration procedure.")
             await self.send_response(service='EiRegisterParty')
             await self.create_party_registration()
-            if self.reports:
-                await self.register_reports(self.reports)
 
         elif response_type == 'oadrDistributeEvent':
             if 'events' in response_payload and len(response_payload['events']) > 0:
+                logger.info(f"The payload tyupe {type(response_payload)}")
                 await self._on_event(response_payload)
 
         elif response_type == 'oadrUpdateReport':
@@ -854,6 +923,10 @@ class OpenADRClient:
                 for report in response_payload['reports']:
                     await self.register_report(report)
 
+        elif response_type == 'oadrCancelPartyRegistration':
+            logger.info("The VTN required us to cancel the registration. Calling the cancel party registration procedure.")
+            await self.on_cancel_party_registration(response_payload)
+
         else:
             logger.warning(f"No handler implemented for incoming message "
                            f"of type {response_type}, ignoring.")
